# plugs/op.py
#
#

""" for op to work for a user the user must have the channel name in his/hers
 status .. use !user-addstatus <username> #channel
 normally the bot doesnt op nicks that join after a split to prevent floods, 
 this can be disabled by using ops-cfg oponsplit 1
"""

__copyright__ = 'this file is in the public domain'

from gozerbot.generic import getwho, rlog
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.callbacks import callbacks
from gozerbot.users import users
from gozerbot.plughelp import plughelp
from gozerbot.persist.persistconfig import PersistConfig
import time

plughelp.add('ops', 'op with command or op on join')

cfg = PersistConfig()
cfg.define('oponsplit', 0)

def opjoincb(bot, ievent):
    """ see if we should op a user on join """
    # don't try to op the bot
    if ievent.nick == bot.nick:
        return
    chan = ievent.channel.lower()
    if chan in bot.state['no-op']:
        return
    time.sleep(1)
    if cfg.get('oponsplit') or ievent.nick.lower() not in bot.splitted:
        if users.status(ievent.userhost, ievent.channel.upper()):
            if chan not in bot.state['opchan']:
                return
            bot.doop(chan, ievent.nick)
    else:
        bot.splitted.remove(ievent.nick.lower())

callbacks.add('JOIN', opjoincb, threaded=True)

def handle_op1(bot, ievent):
    """ op [<nick>] .. op an user """
    chan = ievent.channel.lower()
    if chan in bot.state['no-op']:
        ievent.reply('opping is disabled in %s' % ievent.channel)
        return
    if chan not in bot.state['opchan']:
        ievent.reply("i'm not op in %s" % ievent.channel)
        return
    try:
        nick = ievent.args[0]
    except IndexError:
        nick = ievent.nick
    userhost = getwho(bot, nick)
    if not userhost:
        userhost = ievent.userhost
    if users.status(userhost, ievent.channel.upper()):
        bot.doop(chan, nick)
    else:
        ievent.reply("%s doesn't have %s status" % (nick, \
ievent.channel.upper()))

cmnds.add('op', handle_op1, 'USER', threaded=True, speed=9)
examples.add('op', 'op [<nick>] .. give ops to <nick> or op the person \
giving the command', '1) op 2) op dunker')

def handle_splitted(bot, ievent):
    """ splitted .. show splitted list """
    ievent.reply(str(bot.splitted))

cmnds.add('splitted', handle_splitted, 'OPER')
examples.add('splitted', 'show whos on the splitted list', 'splitted')

def handle_splittedclear(bot, ievent):
    """ splitted-clear .. clear splitted list """
    bot.splitted = []
    ievent.reply('done')

cmnds.add('splitted-clear', handle_splittedclear, 'OPER')
examples.add('splitted-clear', 'clear the splitted list', 'splitted-clear')

def handle_opsdisable(bot, ievent):
    """ disable opping in channel """
    try:
        chan = ievent.args[0].lower()
    except:
        chan = ievent.channel.lower()
    oplist = bot.state['no-op']
    if chan not in oplist:
        bot.state['no-op'].append(chan)
        bot.state.save()
        ievent.reply('opping in %s disabled' % chan)
        if chan in bot.state['opchan']:
            bot.delop(chan, bot.nick)
    else:
        ievent.reply('opping %s is already disabled' % chan)

cmnds.add('ops-disable', handle_opsdisable, 'OPER')
examples.add('ops-disable', 'ops-disable [<channel>] .. disable opping in \
provided channel or the channel command was given in', '1) ops-disable 2) \
ops-disable #dunkbots')

def handle_opsenable(bot, ievent):
    """ enable opping in channel """
    try:
        chan = ievent.args[0].lower()
    except:
        chan = ievent.channel.lower()
    oplist = bot.state['no-op']
    if chan in oplist:
        bot.state['no-op'].remove(chan)
        bot.state.save()
        ievent.reply('opping in %s is enabled' % chan)
    else:
        ievent.reply('opping in %s is already enabled' % chan)

cmnds.add('ops-enable', handle_opsenable, 'OPER')
examples.add('ops-enable', 'ops-enable [<channel>] .. enable opping in \
provided channel or the channel command was given in', '1) ops-enable 2) \
ops-enable #dunkbots')

def handle_opsnoop(bot, ievent):
    """ show in which channels opping is disabled """
    ievent.reply('opping is disabled in %s' % bot.state['no-op'])
    
cmnds.add('ops-list', handle_opsnoop, 'OPER')
examples.add('ops-list', 'list in which channels opping is disabled', \
'ops-list')

def checkmode(bot, ievent):
    """ check mode string """
    plus = 0
    teller = 0
    try:
        args = ievent.arguments
        chan = args[0].lower()
        modestr = args[1]
        who = args[2:]
    except:
        rlog(10, 'op', 'unknow mode string format: %s' % str(ievent))
        return
    for i in modestr:
        if i == '+':
            plus = 1
            continue
        if i == '-':
            plus = 0
            continue
        if i == 'o' and plus:
            if who[teller].lower() == bot.nick.lower():
                if ievent.channel in bot.state['no-op']:
                    bot.delop(ievent.channel.lower(), bot.nick)
                else:
                    rlog(1, 'irc', 'opped on %s' % chan)
                    bot.state['opchan'].append(chan)
        if i == 'o' and not plus:
            if who[teller].lower() == bot.nick.lower():
                rlog(1, 'irc', 'deopped on %s' % chan)
                try:
                    bot.state['opchan'].remove(chan)
                except ValueError:
                    pass
        teller += 1

callbacks.add('MODE', checkmode)



# plugs/shakespear.py
#
#

""" uses the random lib """

__copyright__ = 'this file is in the public domain'
__revision__ = '$Id: shakespeare.py 517 2006-12-21 05:00:00Z deck $'

from gozerbot.generic import handle_exception
from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import re, random

plughelp.add('shakespear', 'display a shakespearean insult')

set_a=["Away I say",
       "Bathe thyself",
       "Be not deaf",
       "Behold thy mirror",
       "Beware my sting",
       "Clean thine ears",
       "Drink up eisel",
       "Eat a crododile",
       "Eat my knickers",
       "Fie upon thee",
       "Forsooth say I",
       "Get thee gone",
       "Get thee hence",
       "Grow unsightly warts",
       "Hear me now",
       "Hear this pox alert",
       "I'll see thee hang'd",
       "Kiss my codpiece",
       "Lead apes in hell",
       "Methinks you stinks",
       "My finger in thine eye",
       ">>Phui<< I say",
       "Remove thine ass hence",
       "Resign not thy day gig",
       "Sit thee on a spit",
       "Sorrow on thee",
       "Swim with leeches",
       "Thou dost intrude",
       "Thy mother wears armor",
       "Trip on thy sword",
       "Tune thy lute",
       "Why, how now putz",
       "Wipe thy ugly face"]
set_b=["artless",
       "bawdy",
       "beslubbering",
       "bootless",
       "cankerous",
       "churlish",
       "cockered",
       "clouted",
       "craven",
       "currish",
       "dankish",
       "dissembling",
       "droning",
       "errant",
       "fawning",
       "fobbing",
       "fool-born",
       "froward",
       "frothy",
       "gleeking",
       "goatish",
       "gorbellied",
       "ill-nurtured",
       "impertinent",
       "incestuous",
       "incurable",
       "infectious",
       "jarring",
       "loggerheaded",
       "lumpish",
       "loutish",
       "mammering",
       "mangled",
       "mewling",
       "paunchy",
       "pribbling",
       "puking",
       "puny",
       "qualling",
       "rank",
       "reeky",
       "roguish",
       "rump-fed",
       "ruttish",
       "saucy",
       "spleeny",
       "spongy",
       "surly",
       "tardy-gaited",
       "tottering",
       "unmuzzled",
       "vain",
       "venomed",
       "warped",
       "wayward",
       "weedy",
       "whoreson",
       "wretched",
       "yeasty"]
set_c=["addlepated",
       "base-court",
       "bat-fowling",
       "beef-witted",
       "beetle-headed",
       "boil-brained",
       "clapper-clawed",
       "clay-brained",
       "codpiece-sniffing",
       "common-kissing",
       "crook-pated",
       "dismal-dreaming",
       "dizzy-eyed",
       "doghearted",
       "dread-bolted",
       "earth-vexing",
       "elf-skinned",
       "fat-kidneyed",
       "fen-sucked",
       "flap-mouthed",
       "fly-bitten",
       "folly-fallen",
       "fool-born",
       "foul-practicing",
       "full-gorged",
       "guts-griping",
       "half-faced",
       "hasty-witted",
       "hedge-born",
       "hell-hated",
       "idle-headed",
       "ill-breeding",
       "ill-nurtured",
       "knotty-pated",
       "mad-brained",
       "milk-livered",
       "motley-minded",
       "onion-eyed",
       "plume-plucked",
       "pottle-deep",
       "pox-marked",
       "reeling-ripe",
       "rough-hewn",
       "rude-growing",
       "rump-fed",
       "shard-borne",
       "sheep-biting",
       "spur-galled",
       "swag-bellied",
       "tardy-gaited",
       "tickle-brained",
       "toad-spotted",
       "unchin-snouted",
       "weather-bitten"]
set_d=["apple-john",
       "baggage",
       "barnacle",
       "bladder",
       "boar-pig",
       "bugbear",
       "bum-bailey",
       "canker-blossom",
       "clack-dish",
       "clotpole",
       "coxcomb",
       "codpiece",
       "death-token",
       "dewberry",
       "dotard",
       "flap-dragon",
       "flax-wench",
       "flea",
       "flirt-gill",
       "foot-licker",
       "fustilarian",
       "giglet",
       "gudgeon",
       "haggard",
       "harpy",
       "hedge-pig",
       "horn-beast",
       "hugger-mugger",
       "jolthead",
       "knave",
       "lewdster",
       "lout",
       "maggot-pie",
       "malt-worm",
       "mammet",
       "measle",
       "minnow",
       "miscreant",
       "moldwarp",
       "mumble-news",
       "nit",
       "nut-hook",
       "pigeon-egg",
       "pignut",
       "pumpion",
       "puttock",
       "ratsbane",
       "rudesby",
       "scut",
       "skainsmate",
       "strumpet",
       "varlot",
       "vassal",
       "wagtail",
       "water-fly",
       "whey-face",
       "winter-cricket"]

def handle_insult(bot, ievent):
    ievent.reply(random.choice(set_a)+" "+random.choice(set_b)+" "+random.choice(set_c)+" "+random.choice(set_d))
    
cmnds.add('insult', handle_insult, 'USER')
examples.add('insult', 'show a shakespearean insult', 'insult')



# gozerplugs/lart.py
#
#

from gozerbot.commands import cmnds
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.aliases import aliasset
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import random

plughelp.add('lart', 'do the lart')

cfg = PersistConfig()
cfg.define('lartlist', [])

def handle_lart(bot, ievent):
    try:
        who = ievent.args[0]
    except IndexError:
        ievent.missing('<who>')
        return
    try:
        txt = random.choice(cfg.get('lartlist'))
    except IndexError:
        ievent.reply('lart list is empty .. use lart-add to add entries .. use "<who>" as a nick holder')
        return
    txt = txt.replace('<who>', who)
    ievent.reply(txt)

cmnds.add('lart', handle_lart, 'USER')
examples.add('lart', 'echo a lart message', 'lart dunker')
aliasset('lart-add', 'lart-cfg lartlist add')
aliasset('lart-del', 'lart-cfg lartlist remove')



from gozerbot.commands import cmnds
from gozerbot.examples import examples
from gozerbot.plughelp import plughelp
import os, string, random

plughelp.add('dope', 'for all the junkies out there')

joint = '''
purple haze
white widow
silver haze
mango haze
krystal
mossad
mexican haze
northern lights
master kush
california orange bud
b-52
binnenkweek mix
skunk
jamaican
jack herrer
tafrolt
pakistaan
primero
red libanon
hija
super hija
afghaan
zwarte afghaan
twizla
malana cream
caramello balls
nepal
maroc
super maroc
charras malana
royal cream
'''.strip().splitlines()

joint_action = '''
rolls a nice %s
passes the %s
gives a large %s joint
puts some %s in the took
fixes a djonko with %s
rolls a blunt with %s
stuffs a waterpipe with %s
smokes %s and gets really high
passes the %s to tashiro
builds a tulp with %s
bakes a %s cake
pulls a %s tea
'''.strip().splitlines()


def handle_joint(bot, ievent):
    """ get a joint  """
    what = random.choice(joint_action) % random.choice(joint)
    bot.action(ievent.channel, what)

cmnds.add('joint', handle_joint, 'USER')
examples.add('joint', 'get a joint', 'joint')




# plugs/dns.py
#
# 

__copyright__ = 'this file is in the public domain'

from gozerbot.commands import cmnds
from gozerbot.examples import examples 
from gozerbot.plughelp import plughelp
import copy
import re
import socket

plughelp.add('dns', 'do ip or host lookup')

_re_hexip = re.compile('^[\da-f]{8}$', re.I)

def handle_hostname(bot, ievent):
    """ hostname <ipnr> .. get hostname of ip number"""
    try:
        item = ievent.args[0]
    except IndexError:
        ievent.missing('<ipnr>')
        return
    try:
        hostname = socket.gethostbyaddr(item)
        ievent.reply(hostname[0])
    except:
        ievent.reply("can't match " + str(item))

cmnds.add('host', handle_hostname, 'USER')
examples.add('host', 'get hostname for ip', 'host 194.109.129.219')

def handle_ip(bot, ievent):
    """ ip <hostname> .. get ip of hostname """
    try:
        item = ievent.args[0]
    except IndexError:
        ievent.missing('<hostname>')
        return
    try:
        ipnr = socket.gethostbyname(item)
        ievent.reply(ipnr)
    except:
        ievent.reply("can't match " + str(item))

cmnds.add('ip', handle_ip, 'USER')
examples.add('ip', 'show ip number of host', 'ip gozerbot.org')

def handle_dns(bot, ievent):
    """ <host|ip> performs a DNS lookup an ip or hostname. """
    if not ievent.args:
        ievent.missing('<host | ip>')
    else:
        is_a   = None
        result = None
        # If we support IPv6 ...
        if socket.has_ipv6:
            # ... then check if this is an IPv6 ip
            try:
                socket.inet_pton(socket.AF_INET6, ievent.args[0])
                is_a = 'ipv6'
            except socket.error:
                pass
        # Ah not an IPv6 ip ...
        if not is_a:
            # ... maybe IPv4 ?
            try:
                socket.inet_pton(socket.AF_INET, ievent.args[0])
                is_a = 'ipv4'
            except socket.error:
                pass
        # Not an ip, must be a hostname then
        if not is_a:
            is_a = 'host'
        # If it was an ip ...
        if is_a in ['ipv4', 'ipv6']:
            try:
                # ... try to resolve it
                result = socket.gethostbyaddr(ievent.args[0])
                if result[1]:
                    result = 'primary: %s, aliases: %s' % \
                        (result[0], ', '.join(result[1]))
                else:
                    result = result[0]
                ievent.reply('%s ip %s resolves to %s' % \
                    (is_a, ievent.args[0], result))
            except Exception, e:
                ievent.reply('could not resolve %s address %s: %s' % \
                    (is_a, ievent.args[0], e[1]))
        # Oh it's a host, lets resolve that
        elif is_a == 'host':
            try:
                result = []
                for info in socket.getaddrinfo(ievent.args[0], None):
                    if info[0] in [socket.AF_INET, socket.AF_INET6] and \
                        info[1] == socket.SOCK_STREAM:
                        ip = info[4][0]
                        if not ip in result:
                            result.append(ip)
                if not result:
                    ievent.reply('could not resolve hostname %s: not found' % \
ievent.args[0])
                else:
                    ievent.reply('%s resolves to: %s' % (ievent.args[0], \
', '.join(result)))
            except Exception, e:
                ievent.reply('could not resolve hostname %s: %s' % \
                    (ievent.args[0], e[1]))
        else:
            ievent.reply('lookup failed, no valid data found')

cmnds.add('dns', handle_dns, 'USER')
examples.add('dns', 'look up an ip or hostname', 'dns gozerbot.org')

def handle_hexip(bot, ievent):
    """ <ip|hex ip> returns the reverse of the given argument. """
    if not ievent.args:
        return ievent.missing('<ip | hex ip>')
    is_a = None
    if _re_hexip.match(ievent.args[0]):
        is_a = 'hexip'
    else:
        try:
            socket.inet_pton(socket.AF_INET, ievent.args[0])
            is_a = 'defip'
        except socket.error:
            pass
    if not is_a:
        ievent.missing('<ip | hex ip>')
        return
    if is_a == 'hexip':
        ip = []
        for i in range(4):
            ip.append(str(int(ievent.args[0][i*2:i*2+2], 16)))
        ip = '.'.join(ip)
        nevent = copy.copy(ievent)
        nevent.args = [ip]
        handle_dns(bot, nevent)
    else:
        test = ievent.args[0].split('.')
        ip = 16777216 * int(test[0]) + 65536 * int(test[1]) + 256 * \
int(test[2]) + int(test[3])
        ievent.reply('ip %s = %08x' % (ievent.args[0], ip))

cmnds.add('hexip', handle_hexip, 'USER')
examples.add('hexip', 'return the hex ip notation of an ip, or vice versa', \
'hexip 7F000001')



# Description: tracks when a nick is last seen
# Author: Wijnand 'tehmaze' Modderman
# Website: http://tehmaze.com
# License: BSD

from gozerbot.callbacks import callbacks
from gozerbot.commands import cmnds
from gozerbot.datadir import datadir
from gozerbot.persist.pdod import Pdod
from gozerbot.persist.persistconfig import PersistConfig
from gozerbot.plughelp import plughelp
from gozerbot.examples import examples
import os, time

plughelp.add('seen', 'remember what people said for the last time')

cfg = PersistConfig()
cfg.define('tz', '+0100')

class Seen(Pdod):
    def __init__(self):
        self.datadir = datadir + os.sep + 'plugs' + os.sep + 'seen'
        Pdod.__init__(self, os.path.join(self.datadir, 'seen.data'))

    def handle_seen(self, bot, ievent):
        if not ievent.args:
            ievent.missing('<nick>')
            return
        nick = ievent.args[0].lower()
        if not self.data.has_key(nick):
            alts = [x for x in self.data.keys() if nick in x]
            if alts:
                alts.sort()
                if len(alts) > 10:
                    nums = len(alts) - 10
                    alts = ', '.join(alts[:10]) + ' + %d others' % nums
                else:
                    alts = ', '.join(alts)
                ievent.reply('no logs for %s, however, I remember seeing: %s' % (nick, alts))
            else:
                ievent.reply('no logs for %s' % nick)
        else:
            text = self.data[nick]['text'] and ': %s' % self.data[nick]['text'] or ''
            ievent.reply('%s was last seen on %s at %s, %s%s' % (nick, self.data[nick]['bot'], 
                time.strftime('%a, %d %b %Y %H:%M:%S '+cfg.get('tz'), time.localtime(self.data[nick]['time'])),
                self.data[nick]['what'], text))

    def privmsgcb(self, bot, ievent):
        self.data[ievent.nick.lower()] = {
            'time':    time.time(),
            'text':    ievent.origtxt,
            'bot':     bot.name,
            'channel': ievent.channel,
            'what':    'saying',
            }

    def joincb(self, bot, ievent):
        self.data[ievent.nick.lower()] = {
            'time':    time.time(),
            'text':    '',
            'bot':     bot.name,
            'channel': ievent.channel,
            'what':    'joining %s' % ievent.channel,
            }
    
    def partcb(self, bot, ievent):
        self.data[ievent.nick.lower()] = {
            'time':    time.time(),
            'text':    ievent.txt,
            'bot':     bot.name,
            'channel': ievent.channel,
            'what':    'parting %s' % ievent.channel,
            }
    
    def quitcb(self, bot, ievent):
        self.data[ievent.nick.lower()] = {
            'time':    time.time(),
            'text':    ievent.txt,
            'bot':     bot.name,
            'channel': ievent.channel,
            'what':    'quitting',
            }
    

    def size(self):
        return len(self.data.keys())

def init():
    global seen
    seen = Seen()
    callbacks.add('PRIVMSG', seen.privmsgcb)
    callbacks.add('JOIN', seen.joincb)
    callbacks.add('PART', seen.partcb)
    callbacks.add('QUIT', seen.quitcb)
    cmnds.add('seen', seen.handle_seen, ['USER', 'CLOUD'])
    examples.add('seen', 'show last spoken txt of <nikc>', 'seen dunker')
    return 1

def shutdown():
    global seen
    seen.save()
    del seen

def size():
    global seen
    return seen.size()
